/*
 * ExactSubtreeMatch.h
 *
 *  Created on: 9. 2. 2014
 *      Author: Jan Travnicek
 */

#ifndef _EXACT_SUBTREE_MATCH_H_
#define _EXACT_SUBTREE_MATCH_H_

#include <tree/Tree.h>
#include <tree/ranked/RankedTree.h>
#include <tree/unranked/UnrankedTree.h>

#include <set>

namespace arbology {

namespace exact {

class ExactSubtreeMatch : public tree::VisitableTreeBase::const_promoting_visitor_type {
public:
	ExactSubtreeMatch() {}

	/**
	 * Performs conversion.
	 * @return left regular grammar equivalent to source automaton.
	 */
	static std::set<unsigned> match(const tree::Tree& subject, const tree::Tree& pattern);

	static std::set<unsigned> match(const tree::UnrankedTree& subject, const tree::UnrankedTree& pattern);
	static std::set<unsigned> match(const tree::RankedTree& subject, const tree::RankedTree& pattern);
	static std::set<unsigned> match(const tree::PrefixRankedNotation& subject, const tree::PrefixRankedNotation& pattern);
private:
	static bool matchHelper(const tree::UnrankedNode& subject, const tree::UnrankedNode& pattern);
	static bool matchHelper(const tree::RankedNode& subject, const tree::RankedNode& pattern);

	static void matchInternal(unsigned& index, std::set<unsigned>& occ, const tree::UnrankedNode& subject, const tree::UnrankedNode& pattern);
	static void matchInternal(unsigned& index, std::set<unsigned>& occ, const tree::RankedNode& subject, const tree::RankedNode& pattern);

	void Visit(void*, const tree::UnrankedTree& subject, const tree::UnrankedTree& pattern) const;
	void Visit(void*, const tree::RankedTree& subject, const tree::RankedTree& pattern) const;
	void Visit(void*, const tree::RankedPattern& subject, const tree::RankedPattern& pattern) const;
	void Visit(void*, const tree::PrefixRankedNotation& subject, const tree::PrefixRankedNotation& pattern) const;
	void Visit(void*, const tree::UnrankedPattern& subject, const tree::UnrankedPattern& pattern) const;

	static const ExactSubtreeMatch EXACT_SUBTREE_MATCH;
};

} /* namespace exact */

} /* namespace arbology */

#endif /* _EXACT_SUBTREE_MATCH_H_ */
