/*
 * BoyerMooreHorspool.cpp
 *
 *  Created on: 5. 11. 2014
 *      Author: Radomir Polach, Tomas Pecka
 */

#include "BoyerMooreHorspool.h"

#include <exception/AlibException.h>
#include <string/LinearString.h>
#include <alphabet/Symbol.h>

#include <map>

namespace stringology {

namespace exact {

std::set<unsigned> BoyerMooreHorspool::match(const string::String& subject, const string::String& pattern) {
	std::set<unsigned> data;
	Accept((void*) &data, subject.getData(), pattern.getData(), BoyerMooreHorspool::BOYER_MOORE_HORSPOOL);
	return data;
}

std::set<unsigned> BoyerMooreHorspool::match(const string::LinearString& string, const string::LinearString& pattern)
{
	std::set<unsigned> occ;
	std::map<alphabet::Symbol, size_t> bcs;

	/* Initialization of BCS to the length of the needle. */
	for(const auto& symbol : string.getAlphabet())
		bcs.insert(std::make_pair(symbol, pattern.getContent().size()));

	/* Filling out BCS, ignoring last character. */
	for(size_t i = 0; i < pattern.getContent().size() - 1; i++)
		bcs[pattern.getContent().at(i)] = pattern.getContent().size() - i - 1;

	/*
	for(const auto& kv: bcs)
		std::cout << std::string(kv.first) << " " << kv.second << std::endl;
	for(const auto& s: string.getContent())
		std::cout << std::string(s);std::cout << std::endl;
	*/

	size_t haystack_offset = 0;
	while(haystack_offset + pattern.getContent().size() <= string.getContent().size()) {
		size_t i = pattern.getContent().size();
		while(i > 0 && string.getContent().at(haystack_offset + i - 1) == pattern.getContent().at(i - 1)) {
			i--;
		}

		// Yay, there is match!!!
		if(i == 0) occ.insert(haystack_offset);
		haystack_offset += bcs[string.getContent().at(haystack_offset + pattern.getContent().size() - 1)];
		//std::cout << haystack_offset << std::endl;
	}
	return occ;
}

void BoyerMooreHorspool::Visit(void*, const string::Epsilon&, const string::Epsilon&) const {
	throw exception::AlibException("Unsupported string type Epsilon");
}

void BoyerMooreHorspool::Visit(void* data, const string::LinearString& subject, const string::LinearString& pattern) const {
	std::set<unsigned> & res = *((std::set<unsigned>*) data);
	res = this->match(subject, pattern);
}

void BoyerMooreHorspool::Visit(void*, const string::CyclicString&, const string::CyclicString&) const {
	throw exception::AlibException("Unsupported string type CyclicString");
}

const BoyerMooreHorspool BoyerMooreHorspool::BOYER_MOORE_HORSPOOL;

} /* namespace exact */

} /* namespace stringology */
